@@ -104,7 +104,8 @@ namespace PlansPlant
     QSize sizeHint() const;
     TasksTreeModel* tasks() { return static_cast<TasksTreeModel*>( model() ); }
     void tasks( TasksTreeModel* Tasks );
-    Task* selected_task() { return TasksTreeModel::task_from_index( currentIndex() ); }
+    QModelIndex selected_index() const;
+    Task* selected_task() { return TasksTreeModel::task_from_index( selected_index() ); }
     bool select_task( const Task* NewCurrent = 0 );
     // Get menu actions of this tree
     QAction* task_add_action() const { return TaskAddAction; }
@@ -123,7 +124,8 @@ namespace PlansPlant
     void move_up();
     void move_down();
     void item_clicked( const QModelIndex& Index );
-    void current_changed( const QModelIndex& NewIndex );
+    void selection_changed( const QItemSelection& New );
+    void selection_changed( const QModelIndex& NewIndex );
   protected: 
     QAction* TaskAddAction;
     QAction* TaskEditAction;
@@ -146,6 +148,7 @@ namespace PlansPlant
   public:
     BlockersEditor( TasksTreeModel& Model0, Task* Object0 = 0, QWidget* Parent = 0 );
     const Task::List& blockers() const;
+    QModelIndex selected_blocker() const;
     void supertask( Task* NewSuperTask );
   public slots:
     void add_blocker();
@@ -154,8 +157,10 @@ namespace PlansPlant
     void move_blocker_down();
     bool can_add_blocker( Task& NewBlocker );
     void blockers_pool_switched( bool On );
-    void blockers_pool_current_changed( const QModelIndex& );
-    void blockers_list_current_changed( const QModelIndex& );
+    void blockers_pool_selection_changed( const QItemSelection& New );
+    void blockers_pool_selection_changed( const QModelIndex& NewIndex );
+    void blockers_list_selection_changed( const QItemSelection& New );
+    void blockers_list_selection_changed( const QModelIndex& NewIndex );
   protected:
     TasksTreeModel& Model;
     BlockersListModel* Blockers;
@@ -209,7 +214,8 @@ namespace PlansPlant
     void accept();
     Task* current_task() const { return CurrentTask; }
   protected slots:
-    virtual void current_changed( const QModelIndex& New );
+    virtual void selection_changed( const QItemSelection& New );
+    virtual void selection_changed( const QModelIndex& NewIndex );
   protected:
     Task* CurrentTask;
     TasksTreeWidget* TasksTree;
@@ -221,7 +227,7 @@ namespace PlansPlant
   public:
     ParentSelectDialog( TasksTreeModel& Model0, QWidget* Parent = 0, const Task::List& Blockers0 = Task::List(), Task* ForTask0 = 0, Task* SuperTask0 = 0 );
   protected slots:
-    void current_changed( const QModelIndex& New );
+    void selection_changed( const QModelIndex& New );
   protected:
     Task* ForTask;
     const Task::List& Blockers;
@@ -232,7 +238,7 @@ namespace PlansPlant
   public:
     BlockerSelectDialog( TasksTreeModel& Model0, QWidget* Parent = 0, Task* ForTask0 = 0, Task* Blocker0 = 0 );
   protected slots:
-    void current_changed( const QModelIndex& New );
+    void selection_changed( const QModelIndex& New );
   protected:
     Task* ForTask;
   }; // BlockerSelectDialog
