@@ -28,11 +28,15 @@
 #include "plansplant/widgets.hpp"
 #include "plansplant/export.hpp"
 #ifdef Q_WS_MAEMO_5
+#define PLANSPLANT_NO_BLOCKERS_SPLITTER
+// #define PLANSPLANT_DISABLE_KINETIC_SCROLLER
+#include <QStackedWidget>
+#endif // Q_WS_MAEMO_5
+#ifdef PLANSPLANT_DISABLE_KINETIC_SCROLLER
 #include <QAbstractKineticScroller>
 #include <QScrollBar>
 #include <QStyleFactory>
-#include <QStackedWidget>
-#endif
+#endif // PLANSPLANT_DISABLE_KINETIC_SCROLLER
 
 namespace PlansPlant
 {
@@ -510,7 +514,7 @@ namespace PlansPlant
 	  Task& T = Car->task();
 	  int SubTasksNum = T.subtasks().size();
 	  
-	  if( From > 0 && From < SubTasksNum && To >= 0 && To < SubTasksNum )
+	  if( From >= 0 && From < SubTasksNum && To >= 0 && To < SubTasksNum )
 	    Result = T.move_subtask( From, To );
 	  else
 	    Result = T.move_dependency( From-SubTasksNum, To-SubTasksNum );
@@ -755,7 +759,7 @@ namespace PlansPlant
     connect( Buttons, SIGNAL( rejected() ), SLOT( reject() ) );
     Layout->addWidget( Buttons );
     setLayout( Layout );
-    connect( TasksTree->selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ), SLOT( current_changed( const QModelIndex& ) ) );
+    connect( TasksTree->selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ), SLOT( selection_changed( const QItemSelection& ) ) );
     TasksTree->select_task( CurrentTask );
   } // TaskSelectDialog( TasksTreeModel&, QWidget*, Task* )
   void TaskSelectDialog::accept()
@@ -764,37 +768,44 @@ namespace PlansPlant
     if( CurrentTask ) QDialog::accept();
     else QMessageBox::warning( this, tr( "PlansPlant" ), tr( "Select a task before." ) );
   } // accept()
-  void TaskSelectDialog::current_changed( const QModelIndex& New )
+  void TaskSelectDialog::selection_changed( const QItemSelection& New )
   {
-    Buttons->button( QDialogButtonBox::Ok )->setEnabled( TasksTree->tasks() && TasksTreeModel::task_from_index( New ) );
-  } // current_changed( const QModelIndex&, const QModelIndex& )
+    QModelIndex NewIndex;
+    if( !New.indexes().isEmpty() )
+      NewIndex = New.indexes().front(); // We've got indexes for every column. Take only the first one.
+    selection_changed( NewIndex );
+  } // selection_changed( const QItemSelection& )
+  void TaskSelectDialog::selection_changed( const QModelIndex& NewIndex )
+  {
+    Buttons->button( QDialogButtonBox::Ok )->setEnabled( TasksTree->tasks() && TasksTreeModel::task_from_index( NewIndex ) );
+  } // selection_changed( const QModelIndex& )
 
   ParentSelectDialog::ParentSelectDialog( TasksTreeModel& Model0, QWidget* Parent, const Task::List& Blockers0, Task* ForTask0, Task* SuperTask0 )
     : TaskSelectDialog( Model0, Parent, SuperTask0, tr( "Select new supertask for " ) + ( ForTask0 ? ( '\"' + ForTask0->name() + '\"' ) : tr( "new task" ) ) ),
       ForTask( ForTask0 ), Blockers( Blockers0 )
   {
-    current_changed( TasksTree->currentIndex() );
+    selection_changed( TasksTree->selected_index() );
   } // ParentSelectDialog( TasksTreeModel&, Task&, QWidget*, Task* )
-  void ParentSelectDialog::current_changed( const QModelIndex& New )
+  void ParentSelectDialog::selection_changed( const QModelIndex& NewIndex )
   {
     bool Enable = false;
-    if( Task* NewParent = TasksTreeModel::task_from_index( New ) )
+    if( Task* NewParent = TasksTreeModel::task_from_index( NewIndex ) )
       Enable = !NewParent->check_loop( Blockers, ForTask );
     Buttons->button( QDialogButtonBox::Ok )->setEnabled( Enable );
-  } // current_changed( const QModelIndex& )
+  } // selection_changed( const QModelIndex& )
   BlockerSelectDialog::BlockerSelectDialog( TasksTreeModel& Model0, QWidget* Parent, Task* ForTask0, Task* Blocker0 )
     : TaskSelectDialog( Model0, Parent, Blocker0, tr( "Select new dependency for " ) + ( ForTask0 ? ( '\"' + ForTask0->name() + '\"' ) : tr( "new task" ) ) + '\"' ),
       ForTask( ForTask0 )
   {
-    current_changed( TasksTree->currentIndex() );
+    selection_changed( TasksTree->selected_index() );
   } // BlockerSelectDialog( TasksTreeModel&, Task&, QWidget*, Task* )
-  void BlockerSelectDialog::current_changed( const QModelIndex& New )
+  void BlockerSelectDialog::selection_changed( const QModelIndex& NewIndex )
   {
     bool Enable = false;
-    if( Task* NewBlocker = TasksTreeModel::task_from_index( New ) )
+    if( Task* NewBlocker = TasksTreeModel::task_from_index( NewIndex ) )
       Enable = !ForTask || !ForTask->check_loop( *NewBlocker );
     Buttons->button( QDialogButtonBox::Ok )->setEnabled( Enable );
-  } // current_changed( const QModelIndex& )
+  } // selection_changed( const QModelIndex& )
 
   class BlockersListModel : public QAbstractItemModel
   {
@@ -934,8 +945,7 @@ namespace PlansPlant
     }
   } // move_down( const QModelIndex& )
 
-#ifdef Q_WS_MAEMO_5
-#define PLANSPLANT_NO_BLOCKERS_SPLITTER
+#ifdef PLANSPLANT_DISABLE_KINETIC_SCROLLER
   void disable_kinetic_scroller( QAbstractScrollArea* Widget )
   {
     if( Widget )
@@ -949,7 +959,7 @@ namespace PlansPlant
 	Widget->verticalScrollBar()->setContextMenuPolicy( Qt::PreventContextMenu );
       }
   } // disable_kinetic_scroller( QAbstractScrollArea* )
-#endif // Q_WS_MAEMO_5
+#endif // PLANSPLANT_DISABLE_KINETIC_SCROLLER
   BlockersEditor::BlockersEditor( TasksTreeModel& Model0, Task* Object0, QWidget* Parent )
     : QWidget( Parent ), Model( Model0 ), Blockers( 0 ), Object( Object0 ), SuperTask( 0 )
   {
@@ -965,9 +975,11 @@ namespace PlansPlant
       Act->setShortcutContext( Qt::WidgetShortcut );
 #else // Q_WS_MAEMO_5
       Act->setShortcuts( QList<QKeySequence>() );
-    disable_kinetic_scroller( UI.BlockersList );
     UI.BlockersSplitter->setMinimumHeight( 220 );
 #endif // Q_WS_MAEMO_5
+#ifdef PLANSPLANT_DISABLE_KINETIC_SCROLLER
+    disable_kinetic_scroller( UI.BlockersList );
+#endif // PLANSPLANT_DISABLE_KINETIC_SCROLLER
     if( Object )
     {
       SuperTask = Object->supertask();
@@ -991,13 +1003,25 @@ namespace PlansPlant
     connect( UI.BlockerRemoveButton, SIGNAL( clicked() ), SLOT( remove_blocker() ) );
     connect( UI.BlockerMoveUpButton, SIGNAL( clicked() ), SLOT( move_blocker_up() ) );
     connect( UI.BlockerMoveDownButton, SIGNAL( clicked() ), SLOT( move_blocker_down() ) );
-    connect( UI.BlockersPoolTree->selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ),
-	     SLOT( blockers_pool_current_changed( const QModelIndex& ) ) );
-    connect( UI.BlockersList->selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ),
-	     SLOT( blockers_list_current_changed( const QModelIndex& ) ) );
+    connect( UI.BlockersPoolTree->selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ),
+	     SLOT( blockers_pool_selection_changed( const QItemSelection& ) ) );
+    connect( UI.BlockersList->selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ),
+	     SLOT( blockers_list_selection_changed( const QItemSelection& ) ) );
     connect( UI.BlockersTreeSwitchButton, SIGNAL( toggled( bool ) ), SLOT( blockers_pool_switched( bool ) ) );
   } // BlockersEditor( TasksTreeModel&, Task*, QWidget* )
   const Task::List& BlockersEditor::blockers() const { return Blockers->blockers(); }
+  QModelIndex BlockersEditor::selected_blocker() const
+  {
+    QModelIndex Result;
+    if( UI.BlockersList )
+      if( QItemSelectionModel* SelectionModel = UI.BlockersList->selectionModel() )
+      {
+	QModelIndexList Indexes = SelectionModel->selectedIndexes();
+	if( !Indexes.isEmpty() )
+	  Result = Indexes.front();
+      }
+    return Result;
+  } // selected_blocker() const
   void BlockersEditor::supertask( Task* NewSuperTask )
   {
     if( SuperTask != NewSuperTask )
@@ -1013,17 +1037,17 @@ namespace PlansPlant
 	if( can_add_blocker( *Blocker ) )
 	{
 	  Blockers->add_blocker( *Blocker );
-	  blockers_list_current_changed( UI.BlockersList->currentIndex() );
+	  blockers_list_selection_changed( selected_blocker() );
 	  blockers_pool_switched( UI.BlockersTreeSwitchButton->isChecked() ); // To update buttons
 	}
   } // add_blocker()
   void BlockersEditor::remove_blocker()
   {
     if( Blockers )
-      if( Task* Blocker = static_cast<Task*>( UI.BlockersList->currentIndex().internalPointer() ) )
+      if( Task* Blocker = TasksTreeModel::task_from_index( selected_blocker() ) ) //!< \todo Don't take it from the other model.
       {
 	Blockers->remove_blocker( *Blocker );
-	blockers_list_current_changed( UI.BlockersList->currentIndex() );
+	blockers_list_selection_changed( selected_blocker() );
 	blockers_pool_switched( UI.BlockersTreeSwitchButton->isChecked() ); // To update buttons
       }
   } // remove_blocker()
@@ -1031,16 +1055,16 @@ namespace PlansPlant
   {
     if( Blockers )
     {
-      Blockers->move_up( UI.BlockersList->currentIndex() );
-      blockers_list_current_changed( UI.BlockersList->currentIndex() );
+      Blockers->move_up( selected_blocker() );
+      blockers_list_selection_changed( selected_blocker() );
     }
   } // move_blocker_up()
   void BlockersEditor::move_blocker_down()
   {
     if( Blockers )
     {
-      Blockers->move_down( UI.BlockersList->currentIndex() );
-      blockers_list_current_changed( UI.BlockersList->currentIndex() );
+      Blockers->move_down( selected_blocker() );
+      blockers_list_selection_changed( selected_blocker() );
     }
   } // move_blocker_down()
   bool BlockersEditor::can_add_blocker( Task& NewBlocker )
@@ -1059,21 +1083,35 @@ namespace PlansPlant
   void BlockersEditor::blockers_pool_switched( bool On )
   {
     if( On )
-      blockers_pool_current_changed( UI.BlockersPoolTree->currentIndex() );
+      blockers_pool_selection_changed( UI.BlockersPoolTree->selected_index() );
     else
       UI.BlockerAddButton->setEnabled( false );
   } // blockers_pool_switched( bool )
-  void BlockersEditor::blockers_pool_current_changed( const QModelIndex& NewIndex )
+  void BlockersEditor::blockers_pool_selection_changed( const QItemSelection& New )
+  {
+    QModelIndex NewIndex;
+    if( !New.indexes().isEmpty() )
+      NewIndex = New.indexes().front();
+    blockers_pool_selection_changed( NewIndex );
+  } // blockers_pool_selection_changed( const QItemSelection& )
+  void BlockersEditor::blockers_pool_selection_changed( const QModelIndex& NewIndex )
   {
     bool CanAdd = false;
     if( Task* NewBlocker = TasksTreeModel::task_from_index( NewIndex ) )
       CanAdd = can_add_blocker( *NewBlocker );
     UI.BlockerAddButton->setEnabled( CanAdd );
-  } // blockers_pool_current_changed( const QModelIndex& )
-  void BlockersEditor::blockers_list_current_changed( const QModelIndex& NewIndex )
+  } // blockers_pool_selection_changed( const QModelIndex& )
+  void BlockersEditor::blockers_list_selection_changed( const QItemSelection& New )
   {
-    if( NewIndex != UI.BlockersList->currentIndex() )
-      qDebug() << "Wrong list index:" << NewIndex << UI.BlockersList->currentIndex();
+    QModelIndex NewIndex;
+    if( !New.indexes().isEmpty() )
+      NewIndex = New.indexes().front();
+    blockers_list_selection_changed( NewIndex );
+  } // blockers_list_selection_changed( const QItemSelection& )
+  void BlockersEditor::blockers_list_selection_changed( const QModelIndex& NewIndex )
+  {
+    if( NewIndex != selected_blocker() )
+      qDebug() << "Wrong list index:" << NewIndex << selected_blocker();
     if( NewIndex.isValid() && NewIndex.internalPointer() )
     {
       UI.BlockerRemoveButton->setEnabled( true );
@@ -1086,7 +1124,7 @@ namespace PlansPlant
       UI.BlockerMoveUpButton->setEnabled( false );
       UI.BlockerMoveDownButton->setEnabled( false );
     }
-  } // blockers_list_current_changed( const QModelIndex& )
+  } // blockers_list_selection_changed( const QModelIndex& )
 
   BlockersEditorDialog::EditorWidget::EditorWidget( TasksTreeModel& Model0, Task* Object0, QDialog* Parent )
     : BlockersEditor( Model0, Object0, Parent )
@@ -1334,9 +1372,10 @@ namespace PlansPlant
   TasksTreeWidget::TasksTreeWidget( QWidget* Parent, TasksTreeModel* Model0 ) : QTreeView( Parent )
   {
     setUniformRowHeights( true );
-#ifdef Q_WS_MAEMO_5
+#ifdef PLANSPLANT_DISABLE_KINETIC_SCROLLER
     disable_kinetic_scroller( this );
-#else
+#endif // PLANSPLANT_DISABLE_KINETIC_SCROLLER
+#ifndef Q_WS_MAEMO_5
     setContextMenuPolicy( Qt::ActionsContextMenu );
 #endif
     TaskAddAction = new QAction( QIcon( ":/images/task-new.svg" ), tr( "&Add task..." ), this );
@@ -1373,7 +1412,7 @@ namespace PlansPlant
     addAction( MoveDownAction );
     connect( this, SIGNAL( clicked( const QModelIndex& ) ), SLOT( item_clicked( const QModelIndex& ) ) );
     if( Model0 ) tasks( Model0 );
-    else current_changed( QModelIndex() );
+    else selection_changed( QModelIndex() );
   } // TasksTreeWidget( TasksTreeModel&, QWidget* )
   QSize TasksTreeWidget::sizeHint() const { return QSize( 800, 0 ); }
   void TasksTreeWidget::tasks( TasksTreeModel* Tasks )
@@ -1390,14 +1429,22 @@ namespace PlansPlant
     setColumnWidth( TasksTreeModel::CompletedCol, 43 );
     setAlternatingRowColors( true );
 #endif
-    connect( selectionModel(), SIGNAL( currentChanged( const QModelIndex&, const QModelIndex& ) ), SLOT( current_changed( const QModelIndex& ) ) );
-    current_changed( QModelIndex() );
+    connect( selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ), SLOT( selection_changed( const QItemSelection& ) ) );
+    selection_changed( QModelIndex() );
 #ifndef Q_WS_MAEMO_5
     if( QHeaderView* Header = header() )
       Header->moveSection( 1, 0 );
     else qDebug() << "No header in the tree";
 #endif
   } // tasks( TasksTreeModel* )
+  QModelIndex TasksTreeWidget::selected_index() const
+  {
+    QModelIndex Result;
+    QModelIndexList Selected = selectedIndexes();
+    if( !Selected.isEmpty() )
+      Result = Selected.front();
+    return Result;
+  } // selected_index() const
   bool TasksTreeWidget::select_task( const Task* NewCurrent )
   {
     bool Result = false;
@@ -1466,7 +1513,7 @@ namespace PlansPlant
   } // add_blocker()
   void TasksTreeWidget::remove_blocker()
   {
-    QModelIndex Current = currentIndex();
+    QModelIndex Current = selected_index();
     if( Current.isValid() )
     {
       if( TasksTreeModel* Tasks = tasks() )
@@ -1482,19 +1529,19 @@ namespace PlansPlant
   } // remove_blocker()
   void TasksTreeWidget::move_up()
   {
-    QModelIndex Index = currentIndex();
+    QModelIndex Index = selected_index();
     if( Index.isValid() )
       if( TasksTreeModel* Tasks = tasks() )
 	if( Tasks->move_task( Index.parent(), Index.row(), Index.row()-1 ) )
-	  current_changed( currentIndex() ); // The index row was changed, but no signal emitted.
+	  selection_changed( selected_index() ); // The index row was changed, but no signal emitted.
   } // move_up()
   void TasksTreeWidget::move_down()
   {
-    QModelIndex Index = currentIndex();
+    QModelIndex Index = selected_index();
     if( Index.isValid() )
       if( TasksTreeModel* Tasks = tasks() )
 	if( Tasks->move_task( Index.parent(), Index.row(), Index.row()+1 ) )
-	  current_changed( currentIndex() ); // The index row was changed, but no signal emitted.
+	  selection_changed( selected_index() ); // The index row was changed, but no signal emitted.
   } // move_down()
   void TasksTreeWidget::item_clicked( const QModelIndex& Index )
   {
@@ -1510,9 +1557,18 @@ namespace PlansPlant
 	    Tasks->modified();
 	}
   } // item_clicked( const QModelIndex& )
-  void TasksTreeWidget::current_changed( const QModelIndex& NewIndex )
+  void TasksTreeWidget::selection_changed( const QItemSelection& New )
+  {
+    QModelIndex NewIndex;
+    if( !New.indexes().isEmpty() )
+      NewIndex = New.indexes().front();
+    selection_changed( NewIndex );
+  } // selection_changed( const QItemSelection& )
+  void TasksTreeWidget::selection_changed( const QModelIndex& NewIndex )
   {
     bool TasksActions = false;
+    bool CanMoveUp = false;
+    bool CanMoveDown = false;
     if( TasksTreeModel* Tasks = tasks() )
     {
       TaskAddAction->setEnabled( true );
@@ -1524,19 +1580,24 @@ namespace PlansPlant
 	{
 	  DependencyRemoveAction->setEnabled( true );
 	  if( Task* Dependent = Tasks->task_from_index( Parent ) )
-	    MoveUpAction->setEnabled( CurTask != Dependent->needs().front() );
-	  else
-	    MoveUpAction->setEnabled( false );
-	  MoveDownAction->setEnabled( NewIndex.row() < Tasks->rowCount( Parent )-1 );
+	  {
+	    CanMoveUp = CurTask != Dependent->needs().front();
+	    CanMoveDown = CurTask != Dependent->needs().back();
+	  }
 	}
 	else
 	{
 	  DependencyRemoveAction->setEnabled( false );
-	  MoveUpAction->setEnabled( NewIndex.row() > 0 );
 	  if( Task* Supertask = Tasks->task_from_index( Parent ) )
-	    MoveDownAction->setEnabled( Supertask->subtasks().back() != CurTask );
+	  {
+	    CanMoveUp = Supertask->subtasks().front() != CurTask;
+	    CanMoveDown = Supertask->subtasks().back() != CurTask;
+	  }
 	  else
-	    MoveDownAction->setEnabled( Tasks->roots().back() != CurTask );
+	  {
+	    CanMoveUp = Tasks->roots().front() != CurTask;
+	    CanMoveDown = Tasks->roots().back() != CurTask;
+	  }
 	}
       }
     }
@@ -1545,13 +1606,10 @@ namespace PlansPlant
     TaskEditAction->setEnabled( TasksActions );
     TaskDeleteAction->setEnabled( TasksActions );
     DependencyAddAction->setEnabled( TasksActions );
-    if( !TasksActions )
-    {
-      DependencyRemoveAction->setEnabled( false );
-      MoveUpAction->setEnabled( false );
-      MoveDownAction->setEnabled( false );
-    }
-  } // current_changed( const QModelIndex& )
+    if( !TasksActions ) DependencyRemoveAction->setEnabled( false ); // Don't always enable if we have tasks actions.
+    MoveUpAction->setEnabled( CanMoveUp );
+    MoveDownAction->setEnabled( CanMoveDown );
+  } // selection_changed( const QModelIndex& )
 
   QString MainWindow::basename( const QString& FileName )
   {
